/**
 * @file
 * Webform Autosave behaviors.
 */

(function($, Drupal, debounce) {
  'use strict';
  // Set our primary store.
  var store = {
    csrfToken: null,
    settings: drupalSettings.webformautosave,
    sid: drupalSettings.webformautosave.sid,
    submissionData: {},
  };

  /**
   * Getter for the CSRF Token.
   *
   * @param callback
   */
  var getCsrfToken = function(callback) {
    // Get the CSRF token if not set, then perform the callback.
    if (store.csrfToken === null) {
      $.get(Drupal.url('session/token')).done(function(data) {
        // Set the token to the returned data.
        store.csrfToken = data;
        callback(store.csrfToken);
      });
    } else {
      callback(store.csrfToken);
    }
  };

  /**
   * Getter for the webform submission data.
   */
  var getSubmission = function(csrfToken) {
    var sid = store.sid;
    var webformId = store.settings.webform_id;
    // Perform a PATCH if there is an SID.
    var getUrl =
      '/webformautosave/' + webformId + '/submission/' + sid + '?_format=json';
    $.ajax({
      url: getUrl,
      method: 'GET',
      headers: {
        'Content-Type': 'application/json',
        'X-CSRF-Token': csrfToken,
      },
    }).done(function(response) {
      store.submissionData = response.data;
    });
  };

  /**
   * Updater for the webform submission.
   */
  var updateSubmission = function(csrfToken) {
    var sid = store.sid;
    var webformId = store.settings.webform_id;
    // Perform a PATCH if there is an SID.
    var patchUrl =
      '/webformautosave/' + webformId + '/submission/' + sid + '?_format=json';
    $.ajax({
      url: patchUrl,
      method: 'PATCH',
      headers: {
        'Content-Type': 'application/json',
        'X-CSRF-Token': csrfToken,
      },
      data: JSON.stringify(store.submissionData),
    }).done(function(response) {
      store.submissionData = response.data;
    });
  };

  /**
   * Posts a webform submission.
   */
  var postSubmission = function(csrfToken) {
    // Set our webform ID.
    store.submissionData['webform_id'] = store.settings.webform_id;
    // Perform a PATCH if there is an SID.
    var postUrl = '/webformautosave/submit?_format=json';
    $.ajax({
      url: postUrl,
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        'X-CSRF-Token': csrfToken,
      },
      data: JSON.stringify(store.submissionData),
    }).done(function(response) {
      store.submissionData = response.data;
      store.sid = response.sid;
    });
  };

  /**
   * Debounced action to perform on change.
   */
  var changeAction = debounce(
    function(event) {
      var value = event.target.value;
      var name = event.target.name;
      store.submissionData[name] = value;
      // Patch the submission if the form has an SID.
      if (store.sid) {
        getCsrfToken(function(csrfToken) {
          updateSubmission(csrfToken);
        });
      }
    },
    250,
    false
  );

  /**
   * Setup our default behaviors for the webformautosave module.
   */
  Drupal.behaviors.webformautosave = {
    // eslint-disable-next-line no-unused-vars
    attach: function(context, settings) {
      // Let's bind an input event to our inputs once.
      $('form', context)
        .once('webformAutosaveBehavior')
        .each(function() {
          // Update our stored submission data.
          if (store.sid) {
            // Get the current submission data.
            getCsrfToken(function(csrfToken) {
              getSubmission(csrfToken);
            });
          }
          // Add an input event listener to each input.
          $(this)
            .find('input, select')
            .each(function() {
              $(this).change(changeAction);
            });
        });
    },
  };
})(jQuery, Drupal, Drupal.debounce);
